@@ -61,6 +61,34 @@ interplot(m = m_cyl, var1 = "cyl", var2 = "wt", hist = TRUE) +
   aes(color = "pink") + theme(legend.position="none") +  # geom_line(color = "pink") + 
   geom_hline(yintercept = 0, linetype = "dashed")
 
+## ----fig.width = 7-------------------------------------------------------
+stdCI_plot <- interplot(m = m_wt, var1 = "wt", var2 = "wt", adjCI = FALSE) +
+ggtitle("Marginal Effects with Standard CIs")
+adjCI_plot <- interplot(m = m_wt, var1 = "wt", var2 = "wt", adjCI = TRUE) +
+ggtitle("Marginal Effects with Adjusted CIs")
+
+library(gridExtra)
+grid.arrange(stdCI_plot, adjCI_plot, ncol = 2)
+
+## ----fig.width = 7, warning = FALSE, message = FALSE---------------------
+pew1.w <- read.csv("pew1_w.csv")
+
+m <- glm(formula=meritocracy~ginicnty+income_i+ginicnty:income_i+income_cnty+black_cnty+
+                perc_bush04+pop_cnty+educ_i+age_i+gender_i+unemp_i+union_i+partyid_i+
+                ideo_i+attend_i+survid2006+survid2007+survid2009,
+              data=pew1.w,family=binomial(link="logit"))
+              
+plot_avg <- interplot(m, var1 = "ginicnty",var2 = "income_i", predPro = FALSE) + 
+ggtitle("Average Conditional Effects")
+
+plot_3val <- interplot(m, var1 = "ginicnty",var2 = "income_i", predPro = TRUE, var2_vals = c(min(pew1.w$income_i), max(pew1.w$income_i))) +
+ggtitle("Conditional Predicted Probabilities for \nCitizens with Low and High Incomes") +
+scale_colour_discrete(guide = guide_legend(title = "Income"), labels = c("Low", "High")) + 
+scale_fill_discrete(guide = guide_legend(title = "Income"), labels = c("Low", "High")) +
+theme(legend.position = c(0, .8), legend.justification = c(0, .5))
+
+grid.arrange(plot_avg, plot_3val, ncol = 2)
+
 ## ----fig.width = 5.5-----------------------------------------------------
 # Create a fake dataset of conditional effects
 fake <- rnorm(100, 0, 1)
